const express = require('express');

const activityApp = express();

const libs = `${process.cwd()}/libs`;

const config = require(`${libs}/config`);

const log = require(`${libs}/log`)(module);

const Activity = require(`${libs}/models/activity`);

const response = require(`${libs}/middlewares/response`);

const email = require(`${libs}/middlewares/email`);

const passport = require('passport');

const fileWorker = require('./file.controller.js');

let upload = require('../middlewares/multer.config.js');

const authorized = require(`${libs}/middlewares/authorize.js`);

function emailSyntax(email) {
  const regex = /^(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i;
  return regex.test(email);
}

activityApp.get('/', async (req, res, next) => {
  const page = parseInt(req.query.page) || 1; 
  const pageSize = parseInt(req.query.pageSize) || 5; 
  try {
    const totalCount = await Activity.count();
    const offset = (page - 1) * pageSize;

    const acts = await Activity.findAll({
      offset,
      limit: pageSize,
      order:[
        ['date', 'DESC']]
    });

    res.json({
      page,
      pageSize,
      totalCount,
      data: acts,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'An error occurred' });
  }
});

activityApp.get('/drafts', async (req, res, next) => {
    const page = parseInt(req.query.page) || 1; 
    const pageSize = parseInt(req.query.pageSize) || 5; 
    try {
      const totalCount = await Activity.count({where: {
        is_draft: true
      }});
      const offset = (page - 1) * pageSize;
  
      const acts = await Activity.findAll({
        offset,
        limit: pageSize,
        where: {
          is_draft: true
        },
        order:[
            ['date', 'DESC']]
      });
  
      res.json({
        page,
        pageSize,
        totalCount,
        data: acts,
      });
    } catch (error) {
      console.error(error);
      res.status(500).json({ error: 'An error occurred' });
    }
  });

activityApp.get('/:id', (req, res, next) => {
    Activity.findByPk(req.params.id).then((act) => {
    if (!act) {
      res.statusCode = 404;
      res.json({ msg: "A atividade não está cadastrada" });
    } else {
      req.result = act.toJSON();
      next();
    }
  }).catch(function (err) {
    log.error(err);
    return next(err);
  });
}, response('activity'));


function transformDateFormat(dateString) {
  // Split the date string into day, month, and year components
  var parts = dateString.split('/');

  // Extract day, month, and year values
  var day = parts[0];
  var month = parts[1];
  var year = parts[2];

  // Concatenate the components in "yyyy/mm/dd" format
  var transformedDate = year + '/' + month + '/' + day;

  return transformedDate;
}



activityApp.post('/', passport.authenticate('bearer', { session: false }), authorized('criar atividade'), async (req, res, next) => {
  console.log(req.body);
  let act = await Activity.create({
    id: 0,
    type:req.body.tipo,
    title: req.body.titulo,
    subtitle: req.body.subtitulo,
    date: transformDateFormat(req.body.dataDePostagem),
    authors:req.body.autor,
    text: req.body.texto,
    name_headline: req.body.nome,
    resume_headline: req.body.resumo,
    date_headline: transformDateFormat(req.body.dataAtividade),
    local_headline: req.body.local,
    additional_headline: req.body.informacoes,
    is_draft: req.body.rascunho,
    is_headline: req.body.is_headline
  }).catch(function (err) {
    log.error(err);
    let errors = [];
    for (let errName in err.errors) {
      errors.push(err.errors[errName].message);
    }
    log.error(errors);
    res.statusCode = 400;
    return res.json({ err, errors });
    // handle error;
  });
  if(!act){
    res.statusCode = 400;
    return res;
  }
  req.result = act.toJSON();
  next();
}, response('activity'));

activityApp.put('/:id', passport.authenticate('bearer', { session: false }), authorized('editar atividade'), async (req, res, next) => {
  let act = await Activity.findByPk(req.params.id).catch(function (err) {
    if (err) {
      log.error(err);
      return next({ err });
    }
  })
  if (!act) {
    res.statusCode = 404;
    return next({
      err: {
        message: 'Atividade não encontrada'
      }
    });
  }
  console.log("TEste");
  act.type = req.body.type || act.type;
  act.title = req.body.title || act.title;
  act.subtitle = req.body.subtitle || act.subtitle;
  act.date = req.body.date || act.date;
  act.authors = req.body.autores || act.authors;
  act.text= req.body.text || act.text;
  act.name_headline= req.body.name_headline || act.name_headline;
  act.resume_headline= req.body.resume_headline || act.resume_headline;
  act.date_headline= req.body.date_headline || act.date_headline;
  act.local_headline= req.body.local_headline || act.local_headline;
  act.additional_headline= req.body.additional_headline || act.additional_headline;
  act.is_draft= req.body.is_draft || act.is_draft;
  act.is_headline= req.body.is_headline || act.is_headline;

  act.save().catch(err => {
    if (err) {
      log.error(err);
      return next({ message: 'Erro ao atualizar publicacao' });
    }
  })
  let activity = act.toJSON();
  res.json({ activity: activity});

});

activityApp.delete('/:id', passport.authenticate('bearer', { session: false }), authorized('editar atividade'), async (req, res, next) => {
  await Activity.destroy({where:{id:req.params.id}}).catch(function (err) {
    if (err) {
        log.error(err);
        return next({ err });
    }
});
  return next({ msg: 'Activity Deleted', status: 200 });
});

module.exports = activityApp;
